@@ -38,7 +38,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: idle.h,v 1.6 2003/10/22 20:05:10 ken3 Exp $ */
+/* $Id: idle.h,v 1.4.4.3 2005/12/12 21:23:58 murch Exp $ */
 
 #ifndef IDLE_H
 #define IDLE_H
@@ -58,11 +58,14 @@ typedef void idle_updateproc_t(idle_flags_t flags);
 /* Is IDLE enabled?  Can also do initial setup, if necessary */
 int idle_enabled(void);
 
-/* Setup for IDLE on 'mailbox'.
- * Accepts pointer to a function which reports mailbox updates and/or
+/* Setup for IDLE.
+ * 'proc' is a pointer to a function which reports mailbox updates and/or
  * ALERTs to the client.
  */
-int idle_init(struct mailbox *mailbox, idle_updateproc_t *proc);
+int idle_init(idle_updateproc_t *proc);
+
+/* Start IDLEing on 'mailbox'. */
+void idle_start(struct mailbox *mailbox);
 
 /* Cleanup when IDLE is completed. */
 void idle_done(struct mailbox *mailbox);
