@@ -38,7 +38,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: idled.h,v 1.4 2003/02/13 20:15:24 rjs3 Exp $ */
+/* $Id: idled.h,v 1.4.4.1 2005/12/12 21:23:59 murch Exp $ */
 
 #ifndef IDLED_H
 #define IDLED_H
@@ -61,7 +61,8 @@ typedef struct idle_data_s {
 typedef enum {
     IDLE_INIT,
     IDLE_DONE,
-    IDLE_NOTIFY
+    IDLE_NOTIFY,
+    IDLE_NOOP
 } idle_msg_t;
 
 #endif
