@@ -38,7 +38,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: mailbox.c,v 1.147.2.30 2005/11/10 06:04:55 murch Exp $
+ * $Id: mailbox.c,v 1.147.2.34 2006/05/23 13:09:37 murch Exp $
  *
  */
 
@@ -84,7 +84,6 @@
 #include "seen.h"
 #include "util.h"
 #include "xmalloc.h"
-#include "byteorder64.h"
 
 static int mailbox_doing_reconstruct = 0;
 #define zeromailbox(m) { memset(&m, 0, sizeof(struct mailbox)); \
@@ -898,7 +897,7 @@ int mailbox_read_index_header(struct mailbox *mailbox)
     mailbox->record_size =
 	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_RECORD_SIZE)));
 
-    if ((mailbox->start_offset < OFFSET_LEAKED_CACHE+sizeof(bit32)) ||
+    if ((mailbox->start_offset < OFFSET_HIGHESTMODSEQ+4) ||
 	(mailbox->record_size < INDEX_RECORD_SIZE)) {
 	if (mailbox_upgrade_index(mailbox))
 	    return IMAP_IOERROR;
@@ -920,10 +919,11 @@ int mailbox_read_index_header(struct mailbox *mailbox)
         mailbox->quota_mailbox_used =
             ntohll(*((bit64 *)(mailbox->index_base+OFFSET_QUOTA_MAILBOX_USED64)));
     } else
-#else
-    mailbox->quota_mailbox_used =
-	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_QUOTA_MAILBOX_USED)));
 #endif
+    {
+	mailbox->quota_mailbox_used =
+	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_QUOTA_MAILBOX_USED)));
+    }
 
     mailbox->pop3_last_login =
 	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_POP3_LAST_LOGIN)));
@@ -936,10 +936,19 @@ int mailbox_read_index_header(struct mailbox *mailbox)
     mailbox->flagged = 
 	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_FLAGGED)));
     mailbox->dirty = 0;
-    mailbox->pop3_new_uidl = !mailbox->exists ||
-	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_POP3_NEW_UIDL)));
+    mailbox->options =
+	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_MAILBOX_OPTIONS)));
     mailbox->leaked_cache_records =
 	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_LEAKED_CACHE)));
+#ifdef HAVE_LONG_LONG_INT
+    mailbox->highestmodseq =
+	ntohll(*((bit64 *)(mailbox->index_base+OFFSET_HIGHESTMODSEQ_64)));
+#else
+    mailbox->highestmodseq =
+	ntohl(*((bit32 *)(mailbox->index_base+OFFSET_HIGHESTMODSEQ)));
+#endif
+
+    if (!mailbox->exists) mailbox->options |= OPT_POP3_NEW_UIDL;
 
     if (!mailbox_doing_reconstruct &&
 	(mailbox->minor_version < MAILBOX_MINOR_VERSION)) {
@@ -986,8 +995,12 @@ struct index_record *record;
     }
     record->content_lines = htonl(*((bit32 *)(buf+OFFSET_CONTENT_LINES)));
     record->cache_version = htonl(*((bit32 *)(buf+OFFSET_CACHE_VERSION)));
-
     message_uuid_unpack(&record->uuid, buf+OFFSET_MESSAGE_UUID);
+#ifdef HAVE_LONG_LONG_INT
+    record->modseq = htonll(*((bit64 *)(buf+OFFSET_MODSEQ_64)));
+#else
+    record->modseq = htonl(*((bit32 *)(buf+OFFSET_MODSEQ)));
+#endif
     return 0;
 }
 
@@ -1298,11 +1311,20 @@ int mailbox_write_index_header(struct mailbox *mailbox)
     *((bit32 *)(buf+OFFSET_DELETED)) = htonl(mailbox->deleted);
     *((bit32 *)(buf+OFFSET_ANSWERED)) = htonl(mailbox->answered);
     *((bit32 *)(buf+OFFSET_FLAGGED)) = htonl(mailbox->flagged);
-    *((bit32 *)(buf+OFFSET_POP3_NEW_UIDL)) = htonl(mailbox->pop3_new_uidl);
+    *((bit32 *)(buf+OFFSET_MAILBOX_OPTIONS)) = htonl(mailbox->options);
     *((bit32 *)(buf+OFFSET_LEAKED_CACHE)) =
 	htonl(mailbox->leaked_cache_records);
+#ifdef HAVE_LONG_LONG_INT
+    *((bit64 *)(buf+OFFSET_HIGHESTMODSEQ_64)) = htonll(mailbox->highestmodseq);
+#else
+    /* zero the unused 32bits */
+    *((bit32 *)(buf+OFFSET_HIGHESTMODSEQ_64)) = htonl(0);
+    *((bit32 *)(buf+OFFSET_HIGHESTMODSEQ)) = htonl(mailbox->highestmodseq);
+#endif
+    *((bit32 *)(buf+OFFSET_SPARE0)) = htonl(0); /* RESERVED */
     *((bit32 *)(buf+OFFSET_SPARE1)) = htonl(0); /* RESERVED */
     *((bit32 *)(buf+OFFSET_SPARE2)) = htonl(0); /* RESERVED */
+    *((bit32 *)(buf+OFFSET_SPARE3)) = htonl(0); /* RESERVED */
 
     if (mailbox->start_offset < header_size)
 	header_size = mailbox->start_offset;
@@ -1342,6 +1364,13 @@ void mailbox_index_record_to_buf(struct index_record *record, char *buf)
     *((bit32 *)(buf+OFFSET_CONTENT_LINES)) = htonl(record->content_lines);
     *((bit32 *)(buf+OFFSET_CACHE_VERSION)) = htonl(record->cache_version);
     message_uuid_pack(&record->uuid, buf+OFFSET_MESSAGE_UUID);
+#ifdef HAVE_LONG_LONG_INT
+    *((bit64 *)(buf+OFFSET_MODSEQ_64)) = htonll(record->modseq);
+#else
+    /* zero the unused 32bits */
+    *((bit32 *)(buf+OFFSET_MODSEQ_64)) = htonl(0);
+    *((bit32 *)(buf+OFFSET_MODSEQ)) = htonl(record->modseq);
+#endif
 }
 
 /*
@@ -1512,6 +1541,15 @@ static void mailbox_upgrade_index_work(struct mailbox *mailbox,
 	*((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED64)) = htonl(0);
     }
 
+    if (ntohl(*((bit32 *)(buf+OFFSET_MINOR_VERSION))) < 8) {
+#ifdef HAVE_LONG_LONG_INT
+	*((bit64 *)(buf+OFFSET_HIGHESTMODSEQ_64)) = htonll(1);
+#else
+	*((bit32 *)(buf+OFFSET_HIGHESTMODSEQ_64)) = htonl(0);
+	*((bit32 *)(buf+OFFSET_HIGHESTMODSEQ)) = htonl(1);
+#endif
+    }
+
     /* change version number */
     *((bit32 *)(buf+OFFSET_MINOR_VERSION)) = htonl(MAILBOX_MINOR_VERSION);
 
@@ -1545,14 +1583,20 @@ static void mailbox_upgrade_index_work(struct mailbox *mailbox,
 
 	calculate_flagcounts = 1;
     }
-    if (oldstart_offset < OFFSET_POP3_NEW_UIDL-quota_offset+sizeof(bit32)) {
-	*((bit32 *)(buf+OFFSET_POP3_NEW_UIDL)) = htonl(!exists);
+    if (oldstart_offset < OFFSET_MAILBOX_OPTIONS-quota_offset+sizeof(bit32)) {
+	unsigned long options = !exists ? OPT_POP3_NEW_UIDL : 0;
+	*((bit32 *)(buf+OFFSET_MAILBOX_OPTIONS)) = htonl(options);
     }
-    if (oldstart_offset < OFFSET_LEAKED_CACHE-quota_offset+sizeof(bit32)) {
-	*((bit32 *)(buf+OFFSET_LEAKED_CACHE)) = htonl(0);
+#if 0
+    if (oldstart_offset < OFFSET_HIGHESTMODSEQ-quota_offset+sizeof(bit32) ||
+	!ntohll(*((bit64 *)(buf+OFFSET_HIGHESTMODSEQ_64)))) {
+	*((bit64 *)(buf+OFFSET_HIGHESTMODSEQ_64)) = htonll(1);
     }
+#endif
+    *((bit32 *)(buf+OFFSET_SPARE0)) = htonl(0); /* RESERVED */
     *((bit32 *)(buf+OFFSET_SPARE1)) = htonl(0); /* RESERVED */
     *((bit32 *)(buf+OFFSET_SPARE2)) = htonl(0); /* RESERVED */
+    *((bit32 *)(buf+OFFSET_SPARE3)) = htonl(0); /* RESERVED */
 
     /* Write new header */
     fwrite(buf, 1, INDEX_HEADER_SIZE, newindex);
@@ -1595,6 +1639,16 @@ static void mailbox_upgrade_index_work(struct mailbox *mailbox,
             if (oldrecord_size < OFFSET_MESSAGE_UUID+MESSAGE_UUID_PACKED_SIZE)
                 memset(buf+OFFSET_MESSAGE_UUID, 0, MESSAGE_UUID_PACKED_SIZE);
 
+	    /* Set the initial modseq to 1 */
+	    if (oldrecord_size < OFFSET_MODSEQ+4) {
+#ifdef HAVE_LONG_LONG_INT
+		*((bit64 *)(buf+OFFSET_MODSEQ_64)) = htonll(1);
+#else
+		*((bit32 *)(buf+OFFSET_MODSEQ_64)) = htonl(0);
+		*((bit32 *)(buf+OFFSET_MODSEQ)) = htonl(1);
+#endif
+	    }
+
 	    fwrite(buf+oldrecord_size, recsize_diff, 1, newindex);
 	}
     }
@@ -2402,6 +2456,11 @@ int mailbox_expunge(struct mailbox *mailbox,
 	}
     }
 
+    if (numdeleted > 0) {
+	syslog(LOG_NOTICE, "Expunged %d messages from %s",
+	       numdeleted, mailbox->name);
+    }
+
     if (deleted) free(deleted);
 
     return 0;
@@ -2570,7 +2629,9 @@ int mailbox_create(const char *name,
     mailbox.deleted = 0;
     mailbox.answered = 0;
     mailbox.flagged = 0;
-    mailbox.pop3_new_uidl = 1;
+    mailbox.options = OPT_POP3_NEW_UIDL;
+    mailbox.leaked_cache_records = 0;
+    mailbox.highestmodseq = 1;
 
     if (!uniqueid) {
 	size_t unique_size = sizeof(char) * 32;
@@ -2694,6 +2755,7 @@ int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
     } while (mailbox->mpath && (path != mailbox->mpath) &&
 	     (path = mailbox->mpath));
 
+    syslog(LOG_NOTICE, "Deleted mailbox %s", mailbox->name);
     mailbox_close(mailbox);
     return 0;
 }
