@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: mboxlist.c,v 1.221.2.21 2005/12/13 19:36:03 murch Exp $
+ * $Id: mboxlist.c,v 1.221.2.22 2006/05/22 20:37:25 murch Exp $
  */
 
 #include <config.h>
@@ -477,10 +477,6 @@ mboxlist_mycreatemailboxcheck(char *name,
 		free(acl);
 		return IMAP_PERMISSION_DENIED;
 	    }
-	    /* disallow wildcards in userids with inboxes. */	     
-	    if (strchr(mbox, '*') || strchr(mbox, '%') || strchr(mbox, '?')) {
-		return IMAP_MAILBOX_BADNAME;
-	    }
 
 	    /*
 	     * Users by default have all access to their personal mailbox(es),
