@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.144.2.39 2006/02/01 19:25:38 murch Exp $
+ * $Id: pop3d.c,v 1.144.2.42 2006/05/22 19:38:32 murch Exp $
  */
 #include <config.h>
 
@@ -204,6 +204,10 @@ static int popd_canon_user(sasl_conn_t *conn, void *context,
 
     if (config_getswitch(IMAPOPT_POPSUBFOLDERS)) {
 	/* make a working copy of the auth[z]id */
+	if (ulen > MAX_MAILBOX_NAME) {
+	    sasl_seterror(conn, 0, "buffer overflow while canonicalizing");
+	    return SASL_BUFOVER;
+	}
 	memcpy(userbuf, user, ulen);
 	userbuf[ulen] = '\0';
 	user = userbuf;
@@ -261,6 +265,10 @@ static int popd_proxy_policy(sasl_conn_t *conn,
 
 	/* make a working copy of the authzid */
 	if (!rlen) rlen = strlen(requested_user);
+	if (rlen > MAX_MAILBOX_NAME) {
+	    sasl_seterror(conn, 0, "buffer overflow while proxying");
+	    return SASL_BUFOVER;
+	}
 	memcpy(userbuf, requested_user, rlen);
 	userbuf[rlen] = '\0';
 	requested_user = userbuf;
@@ -980,7 +988,7 @@ static void cmdloop(void)
 			 popd_msg[msg].deleted) {
 		    prot_printf(popd_out, "-ERR No such message\r\n");
 		}
-		else if (mboxstruct.pop3_new_uidl) {
+		else if (mboxstruct.options & OPT_POP3_NEW_UIDL) {
 			    prot_printf(popd_out, "+OK %u %lu.%u\r\n", msg, 
 					mboxstruct.uidvalidity,
 					popd_msg[msg].uid);
@@ -995,7 +1003,7 @@ static void cmdloop(void)
 		prot_printf(popd_out, "+OK unique-id listing follows\r\n");
 		for (msg = 1; msg <= popd_exists; msg++) {
 		    if (!popd_msg[msg].deleted) {
-			if (mboxstruct.pop3_new_uidl) {
+			if (mboxstruct.options & OPT_POP3_NEW_UIDL) {
 			    prot_printf(popd_out, "%u %lu.%u\r\n", msg, 
 					mboxstruct.uidvalidity,
 					popd_msg[msg].uid);
@@ -1126,15 +1134,18 @@ static void cmd_apop(char *response)
     /* failed authentication */
     if (sasl_result != SASL_OK)
     {
-	sleep(3);      
-		
-	prot_printf(popd_out, "-ERR [AUTH] authenticating: %s\r\n",
-		    sasl_errstring(sasl_result, NULL, NULL));
-
 	syslog(LOG_NOTICE, "badlogin: %s APOP (%s) %s",
 	       popd_clienthost, popd_apop_chal,
 	       sasl_errdetail(popd_saslconn));
 	
+	sleep(3);      
+
+	/* Don't allow user probing */
+	if (sasl_result == SASL_NOUSER) sasl_result = SASL_BADAUTH;
+		
+	prot_printf(popd_out, "-ERR [AUTH] authenticating: %s\r\n",
+		    sasl_errstring(sasl_result, NULL, NULL));
+
 	if (popd_subfolder) {
 	    free(popd_subfolder);
 	    popd_subfolder = 0;
@@ -1406,11 +1417,6 @@ void cmd_auth(char *arg)
 	    break;
 	default:
 	    /* failed authentication */
-	    sleep(3);
-		
-	    prot_printf(popd_out, "-ERR [AUTH] authenticating: %s\r\n",
-			sasl_errstring(sasl_result, NULL, NULL));
-
 	    if (authtype) {
 		syslog(LOG_NOTICE, "badlogin: %s %s %s",
 		       popd_clienthost, authtype,
@@ -1419,6 +1425,14 @@ void cmd_auth(char *arg)
 		syslog(LOG_NOTICE, "badlogin: %s %s",
 		       popd_clienthost, authtype);
 	    }
+
+	    sleep(3);
+
+	    /* Don't allow user probing */
+	    if (sasl_result == SASL_NOUSER) sasl_result = SASL_BADAUTH;
+		
+	    prot_printf(popd_out, "-ERR [AUTH] authenticating: %s\r\n",
+			sasl_errstring(sasl_result, NULL, NULL));
 	}
 	
 	if (popd_subfolder) {
