@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: protocol.h,v 1.5 2004/05/06 18:52:07 ken3 Exp $ */
+/* $Id: protocol.h,v 1.2.2.5 2005/02/21 19:25:44 ken3 Exp $ */
 
 #ifndef _INCLUDED_PROTOCOL_H
 #define _INCLUDED_PROTOCOL_H
@@ -55,13 +55,14 @@ enum {
     CAPA_IDLE		= (1 << 2),
     CAPA_MUPDATE	= (1 << 3),
     CAPA_MULTIAPPEND	= (1 << 4),
+    CAPA_LISTSUBSCRIBED	= (1 << 5),
 
     /* LMTP capabilities */
     CAPA_PIPELINING	= (1 << 2),
     CAPA_IGNOREQUOTA	= (1 << 3)
 };
 
-#define MAX_CAPA 5
+#define MAX_CAPA 6
 
 struct capa_t {
     const char *str;
@@ -89,7 +90,8 @@ struct tls_cmd_t {
 
 struct simple_cmd_t {
     const char *cmd;		/* command string */
-    const char *resp;		/* success response */
+    const char *unsol;		/* unsolicited response */
+    const char *ok;		/* success response */
 };
 
 struct protocol_t {
@@ -109,7 +111,8 @@ enum {
     PROTOCOL_POP3,
     PROTOCOL_NNTP,
     PROTOCOL_LMTP,
-    PROTOCOL_MUPDATE
+    PROTOCOL_MUPDATE,
+    PROTOCOL_CSYNC
 };
 
 #endif /* _INCLUDED_PROTOCOL_H */
