@@ -1,7 +1,7 @@
 /* syncnews.c -- program to synchronize active file with mailbox list
  *
  * 
- * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,7 +42,7 @@
  */
 
 /*
- * $Id: syncnews.c,v 1.20 2002/11/06 20:43:24 rjs3 Exp $
+ * $Id: syncnews.c,v 1.24.2.2 2004/05/25 01:28:13 ken3 Exp $
  */
 #include <config.h>
 
@@ -58,10 +58,9 @@
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <sys/stat.h>
-#include <com_err.h>
 
 #include "assert.h"
-#include "imapconf.h"
+#include "global.h"
 #include "xmalloc.h"
 #include "exitcodes.h"
 #include "imap_err.h"
@@ -107,14 +106,16 @@ int main(int argc, char **argv)
 	}
     }
 
-    config_init(alt_config, "syncnews");
+    cyrus_init(alt_config, "syncnews", 0);
 
     if (!argv[optind] || argv[optind+1]) usage();
 
     readactive(argv[optind]);
     do_syncnews();
 
-    exit(code);
+    cyrus_done();
+
+    return code;
 }
 
 #define GROUPGROW 300
@@ -151,6 +152,7 @@ void readactive(char *active)
     if (!active_file) {
 	perror(active);
 	syslog(LOG_ERR, "cannot read active file %s: %m", active);
+	cyrus_done();
 	exit(EC_NOINPUT);
     }
 
@@ -192,6 +194,7 @@ void readactive(char *active)
     if (ferror(active_file)) {
 	fprintf(stderr, "syncnews: error reading active file\n");
 	syslog(LOG_ERR, "error reading active file");
+	cyrus_done();
 	exit(EC_DATAERR);
     }
     fclose(active_file);
@@ -199,6 +202,7 @@ void readactive(char *active)
     if (group_num == 0) {
 	fprintf(stderr, "syncnews: no groups in active file\n");
 	syslog(LOG_ERR, "no groups in active file");
+	cyrus_done();
 	exit(EC_DATAERR);
     }
 
@@ -208,6 +212,7 @@ void readactive(char *active)
   badactive:
     fprintf(stderr, "syncnews: bad line %d in active file\n", lineno);
     syslog(LOG_ERR, "bad line %d in active file", lineno);
+    cyrus_done();
     exit(EC_DATAERR);
     
 }
@@ -241,7 +246,7 @@ void do_syncnews(void)
 	if (!group_seen[i]) {
 	    r = mboxlist_createmailbox(group[i],
 				       MBTYPE_NETNEWS, "news",
-				       1, "anonymous", 0);
+				       1, "anonymous", NULL, 0, 0, 0);
 
 	    if (r == IMAP_MAILBOX_BADNAME) {
 		printf("ignored %s\n", group[i]);
@@ -263,6 +268,7 @@ void do_syncnews(void)
 void fatal(const char* s, int code)
 {
     fprintf(stderr, "syncnews: %s\n", s);
+    cyrus_done();
     exit(code);
 }
 
