@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: util.c,v 1.28 2004/02/27 18:17:33 ken3 Exp $
+ * $Id: util.c,v 1.27.2.3 2004/09/09 18:47:11 shadow Exp $
  */
 
 #include <config.h>
@@ -285,6 +285,29 @@ int cyrus_close_sock(int fd)
     return close(fd);
 }
 
+void cyrus_reset_stdio()
+{
+    int devnull = open("/dev/null", O_RDWR, 0);
+    
+    if (devnull == -1) {
+        fatal("open() on /dev/null failed", EC_TEMPFAIL);
+    }
+    
+    /* stdin */
+    shutdown(0, SHUT_RD);
+    dup2(devnull, 0);
+    
+    /* stdout */
+    shutdown(1, SHUT_RD);
+    dup2(devnull, 1);
+    
+    /* stderr */
+    shutdown(2, SHUT_RD);
+    dup2(devnull, 2);
+
+    if (devnull > 2) close(devnull);
+}
+
 /* Given a mkstemp(3) pattern for a filename,
  * create the file and return the file descriptor.
  *
