@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: master.c,v 1.98 2004/06/22 19:00:05 rjs3 Exp $ */
+/* $Id: master.c,v 1.85.2.15 2005/04/06 20:37:00 ken3 Exp $ */
 
 #include <config.h>
 
@@ -109,6 +109,8 @@
 
 #include "xmalloc.h"
 
+#include "message_uuid_master.h"
+
 enum {
     become_cyrus_early = 1,
     child_table_size = 10000,
@@ -118,9 +120,14 @@ enum {
 static int verbose = 0;
 static int listen_queue_backlog = 32;
 static int pidfd = -1;
+static int have_uuid = 0;
 
 const char *MASTER_CONFIG_FILENAME = DEFAULT_MASTER_CONFIG_FILENAME;
 
+#define SERVICE_NONE -1
+#define SERVICE_MAX  INT_MAX-10
+#define SERVICENAME(x) ((x) ? x : "unknown")
+
 struct service *Services = NULL;
 int allocservices = 0;
 int nservices = 0;
@@ -152,7 +159,7 @@ struct centry {
     pid_t pid;
     enum sstate service_state;	/* SERVICE_STATE_* */
     time_t janitor_deadline;	/* cleanup deadline */
-    struct service *s;
+    int si;			/* Services[] index */
     struct centry *next;
 };
 static struct centry *ctable[child_table_size];
@@ -342,7 +349,11 @@ void service_create(struct service *s)
 
     if (s->associate > 0)
 	return;			/* service is already activated */
-    
+
+    if (!s->name)
+	fatal("Serious software bug found: service_create() called on unnamed service!",
+		EX_SOFTWARE);
+
     if (s->listen[0] == '/') { /* unix socket */
 	res0_is_local = 1;
 	res0 = (struct addrinfo *)malloc(sizeof(struct addrinfo));
@@ -494,6 +505,8 @@ void service_create(struct service *s)
 	
 	if (s == &service) {
 	    if (nservices == allocservices) {
+		if (allocservices > SERVICE_MAX - 5)
+		    fatal("out of service structures, please restart", EX_UNAVAILABLE);
 		Services = xrealloc(Services, 
 				    (allocservices+=5) * sizeof(struct service));
 		if (!Services) fatal("out of memory", EX_UNAVAILABLE);
@@ -572,7 +585,7 @@ void fcntl_unset(int fd, int flag)
     }
 }
 
-void spawn_service(struct service *s)
+void spawn_service(const int si)
 {
     /* Note that there is logic that depends on this being 2 */
     const int FORKRATE_INTERVAL = 2;
@@ -582,12 +595,21 @@ void spawn_service(struct service *s)
     char path[PATH_MAX];
     static char name_env[100], name_env2[100];
     struct centry *c;
+    struct service * const s = &Services[si];
     time_t now = time(NULL);
-    
+    struct message_uuid uuid_prefix;
+    char *uuid_prefix_text;
+    static char uuid_env[100];
+
+    if (!s->name) {
+	fatal("Serious software bug found: spawn_service() called on unnamed service!",
+		EX_SOFTWARE);
+    }
+
     /* update our fork rate */
     if(now - s->last_interval_start >= FORKRATE_INTERVAL) {
 	int interval;
-	
+
 	s->forkrate = (s->interval_forks/2) + (s->forkrate/2);
 	s->interval_forks = 0;
 	s->last_interval_start += FORKRATE_INTERVAL;
@@ -624,6 +646,21 @@ void spawn_service(struct service *s)
 	return;
     }
 
+    if (s->provide_uuid) {
+        if (!message_uuid_master_next_child(&uuid_prefix)) {
+            syslog(LOG_ERR, "Failed to generate UUID for %s", s->name);
+            message_uuid_set_null(&uuid_prefix);
+        }
+
+        if (!message_uuid_master_checksum(&uuid_prefix)) {
+            syslog(LOG_ERR, "Failed to checksum UUID for %s", s->name);
+            message_uuid_set_null(&uuid_prefix);
+        }
+
+        uuid_prefix_text = message_uuid_text(&uuid_prefix);
+    } else
+        uuid_prefix_text = NULL;
+
     switch (p = fork()) {
     case -1:
 	syslog(LOG_ERR, "can't fork process to run service %s: %m", s->name);
@@ -667,6 +704,13 @@ void spawn_service(struct service *s)
 	snprintf(name_env2, sizeof(name_env2), "CYRUS_ID=%d", s->associate);
 	putenv(name_env2);
 
+	/* add UUID prefix to environment */
+	if (s->provide_uuid) {
+	    snprintf(uuid_env, sizeof(uuid_env), "CYRUS_UUID_PREFIX=%s",
+		     uuid_prefix_text);
+	    putenv(uuid_env);
+	}
+
 	execv(path, s->exec);
 	syslog(LOG_ERR, "couldn't exec %s: %m", path);
 	exit(EX_OSERR);
@@ -681,7 +725,7 @@ void spawn_service(struct service *s)
 	c = get_centry();
 	c->pid = p;
 	c->service_state = SERVICE_STATE_READY;
-	c->s = s;
+	c->si = si;
 	c->next = ctable[p % child_table_size];
 	ctable[p % child_table_size] = c;
 	break;
@@ -693,6 +737,10 @@ void schedule_event(struct event *a)
 {
     struct event *ptr;
 
+    if (! a->name)
+	fatal("Serious software bug found: schedule_event() called on unnamed event!",
+		EX_SOFTWARE);
+
     if (!schedule || a->mark < schedule->mark) {
 	a->next = schedule;
 	schedule = a;
@@ -771,7 +819,7 @@ void spawn_schedule(time_t now)
 		c = get_centry();
 		c->pid = p;
 		c->service_state = SERVICE_STATE_READY;
-		c->s = NULL;
+		c->si = SERVICE_NONE;
 		c->next = ctable[p % child_table_size];
 		ctable[p % child_table_size] = c;
 		
@@ -805,6 +853,7 @@ void reap_child(void)
     int status;
     pid_t pid;
     struct centry *c;
+    struct service *s;
 
     while ((pid = waitpid((pid_t) -1, &status, WNOHANG)) > 0) {
 	if (WIFEXITED(status)) {
@@ -822,6 +871,8 @@ void reap_child(void)
 	while(c && c->pid != pid) c = c->next;
 	
 	if (c && c->pid == pid) {
+	    s = ((c->si) != SERVICE_NONE) ? &Services[c->si] : NULL;
+
 	    /* paranoia */
 	    switch (c->service_state) {
 	    case SERVICE_STATE_READY:
@@ -832,23 +883,23 @@ void reap_child(void)
 	    default:
 		syslog(LOG_CRIT, 
 		       "service %s pid %d in ILLEGAL STATE: exited. Serious software bug or memory corruption detected!",
-		       ((c->s) ? c->s->name : "unknown"), pid);
+		       SERVICENAME(s->name), pid);
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in ILLEGAL state: forced to valid UNKNOWN state",
-		       ((c->s) ? c->s->name : "unknown"), pid);
+		       SERVICENAME(s->name), pid);
 		c->service_state = SERVICE_STATE_UNKNOWN;
 	    }
-	    if (c->s) {
+	    if (s) {
 	        /* update counters for known services */
 		switch (c->service_state) {
 		case SERVICE_STATE_READY:
-		    c->s->nactive--;
-		    c->s->ready_workers--;
+		    s->nactive--;
+		    s->ready_workers--;
 		    if (WIFSIGNALED(status) ||
 			(WIFEXITED(status) && WEXITSTATUS(status))) {
 			syslog(LOG_WARNING, 
 			       "service %s pid %d in READY state: terminated abnormally",
-			       c->s->name, pid);
+			       SERVICENAME(s->name), pid);
 		    }
 		    break;
 		    
@@ -856,35 +907,34 @@ void reap_child(void)
 		    /* uh? either we got duplicate signals, or we are now MT */
 		    syslog(LOG_WARNING, 
 			   "service %s pid %d in DEAD state: receiving duplicate signals", 
-			   c->s->name, pid);
+			   SERVICENAME(s->name), pid);
 		    break;
 		    
 		case SERVICE_STATE_BUSY:
-		    c->s->nactive--;
+		    s->nactive--;
 		    if (WIFSIGNALED(status) ||
 			(WIFEXITED(status) && WEXITSTATUS(status))) {
 			syslog(LOG_DEBUG,
 			       "service %s pid %d in BUSY state: terminated abnormally",
-			       c->s->name, pid);
+			       SERVICENAME(s->name), pid);
 		    }
 		    break;
 		    
 		case SERVICE_STATE_UNKNOWN:
-		    c->s->nactive--;
+		    s->nactive--;
 		    syslog(LOG_WARNING,
 			   "service %s pid %d in UNKNOWN state: exited",
-			   c->s->name, pid);
-		    break;
-
-		default:
-		    /* Prevent Warning */
+			   SERVICENAME(s->name), pid);
 		    break;
 		} 
 	    } else {
-	    	/* children from spawn_schedule (events) or children that messaged us before being registered */
+	    	/* children from spawn_schedule (events) or
+		 * children that messaged us before being registered or
+		 * children of services removed by reread_conf() */
 		if (c->service_state != SERVICE_STATE_READY) {
-		    syslog(LOG_ERR,
-			   "unknown service pid %d in state %d: exited (maybe using a service as an event?)",
+		    syslog(LOG_WARNING,
+			   "unknown service pid %d in state %d: exited (maybe using a service as an event,"
+			   " or a service was removed by SIGHUP?)",
 			   pid, c->service_state);
 		}
 	    }
@@ -899,13 +949,14 @@ void reap_child(void)
 	    c->pid = pid;
 	    c->service_state = SERVICE_STATE_DEAD;
 	    c->janitor_deadline = time(NULL) + 2;
-	    c->s = NULL;
+	    c->si = SERVICE_NONE;
 	    c->next = ctable[pid % child_table_size];
 	    ctable[pid % child_table_size] = c;
 	}
-    if (verbose && c && c->s)
-	syslog(LOG_DEBUG, "service %s now has %d ready workers\n", 
-	       c->s->name, c->s->ready_workers);
+	if (verbose && c && (c->si != SERVICE_NONE))
+	    syslog(LOG_DEBUG, "service %s now has %d ready workers\n", 
+		    SERVICENAME(Services[c->si].name),
+		    Services[c->si].ready_workers);
     }
 }
 
@@ -1052,14 +1103,11 @@ void sighandler_setup(void)
     }
 }
 
-void process_msg(struct service *s, struct notify_message *msg) 
+void process_msg(const int si, struct notify_message *msg) 
 {
-    struct centry * c;
-    
-    /* s must NOT be null
-     * but we don't assert(s) since the current code 
-     * makes NULL s an impossibility anyway 
-     */
+    struct centry *c;
+    /* si must NOT point to an invalid service */
+    struct service * const s = &Services[si];;
 
     /* Search hash table with linked list for pid */
     c = ctable[msg->service_pid % child_table_size];
@@ -1068,10 +1116,10 @@ void process_msg(struct service *s, struct notify_message *msg)
     /* Did we find it? */
     if (!c || c->pid != msg->service_pid) {
 	syslog(LOG_WARNING, "service %s pid %d: while trying to process message 0x%x: not registered yet", 
-	       s->name, msg->service_pid, msg->message);
+	       SERVICENAME(s->name), msg->service_pid, msg->message);
 	/* resilience paranoia. Causes small performance loss when used */
 	c = get_centry();
-	c->s = s;
+	c->si = si;
 	c->pid = msg->service_pid;
 	c->service_state = SERVICE_STATE_UNKNOWN;
 	c->next = ctable[c->pid % child_table_size];
@@ -1079,17 +1127,18 @@ void process_msg(struct service *s, struct notify_message *msg)
     }
     
     /* paranoia */
-    if (s != c->s) {
+    if (si != c->si) {
 	syslog(LOG_ERR, 
 	       "service %s pid %d: changing from service %s due to received message",
-	       s->name, c->pid, ( (c->s) ? c->s->name : "unknown" ));
-	c->s = s;
+	       SERVICENAME(s->name), c->pid,
+	       ((c->si != SERVICE_NONE && Services[c->si].name) ? Services[c->si].name : "unknown"));
+	c->si = si;
     }
     switch (c->service_state) {
     case SERVICE_STATE_UNKNOWN:
 	syslog(LOG_WARNING, 
 	       "service %s pid %d in UNKNOWN state: processing message 0x%x",
-	       s->name, c->pid, msg->message);
+	       SERVICENAME(s->name), c->pid, msg->message);
 	break;
     case SERVICE_STATE_READY:
     case SERVICE_STATE_BUSY:
@@ -1098,10 +1147,10 @@ void process_msg(struct service *s, struct notify_message *msg)
     default:
 	syslog(LOG_CRIT,
 	       "service %s pid %d in ILLEGAL state: detected. Serious software bug or memory corruption uncloaked while processing message 0x%x from child!",
-	       s->name, c->pid, msg->message);
+	       SERVICENAME(s->name), c->pid, msg->message);
 	syslog(LOG_DEBUG,
 	       "service %s pid %d in ILLEGAL state: forced to valid UNKNOWN state",
-	       s->name, c->pid);
+	       SERVICENAME(s->name), c->pid);
 	c->service_state = SERVICE_STATE_UNKNOWN;
 	break;
     }
@@ -1114,7 +1163,7 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    /* duplicate message? */
 	    syslog(LOG_WARNING,
 		   "service %s pid %d in READY state: sent available message but it is already ready",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    break;
 	    
 	case SERVICE_STATE_UNKNOWN:
@@ -1122,7 +1171,7 @@ void process_msg(struct service *s, struct notify_message *msg)
 	     * we don't increment ready_workers */
 	    syslog(LOG_DEBUG,
 		   "service %s pid %d in UNKNOWN state: now available and in READY state",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    c->service_state = SERVICE_STATE_READY;
 	    break;
 	    
@@ -1130,14 +1179,10 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    if (verbose) 
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in BUSY state: now available and in READY state",
-		       s->name, c->pid);
+		       SERVICENAME(s->name), c->pid);
 	    c->service_state = SERVICE_STATE_READY;
 	    s->ready_workers++;
 	    break;
-
-	default:
-	    /* Prevent Warning */
-	    break;
 	}
 	break;
 
@@ -1147,13 +1192,13 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    /* duplicate message? */
 	    syslog(LOG_WARNING,
 		   "service %s pid %d in BUSY state: sent unavailable message but it is already busy",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    break;
 	    
 	case SERVICE_STATE_UNKNOWN:
 	    syslog(LOG_DEBUG,
 		   "service %s pid %d in UNKNOWN state: now unavailable and in BUSY state",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    c->service_state = SERVICE_STATE_BUSY;
 	    break;
 	    
@@ -1161,14 +1206,10 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    if (verbose)
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in READY state: now unavailable and in BUSY state",
-		       s->name, c->pid);
+		       SERVICENAME(s->name), c->pid);
 	    c->service_state = SERVICE_STATE_BUSY;
 	    s->ready_workers--;
 	    break;
-
-	default:
-	    /* Prevent Warning */
-	    break;
 	}
 	break;
 
@@ -1179,7 +1220,7 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    if (verbose)
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in BUSY state: now serving connection",
-		       s->name, c->pid);
+		       SERVICENAME(s->name), c->pid);
 	    break;
 	    
 	case SERVICE_STATE_UNKNOWN:
@@ -1187,22 +1228,18 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    c->service_state = SERVICE_STATE_BUSY;
 	    syslog(LOG_DEBUG,
 		   "service %s pid %d in UNKNOWN state: now in BUSY state and serving connection",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    break;
 	    
 	case SERVICE_STATE_READY:
 	    syslog(LOG_ERR, 
 		   "service %s pid %d in READY state: reported new connection, forced to BUSY state",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    /* be resilient on face of a bogon source, so lets err to the side
 	     * of non-denial-of-service */
 	    c->service_state = SERVICE_STATE_BUSY;
 	    s->nconnections++;
 	    s->ready_workers--;
-
-	default:
-	    /* Prevent Warning */
-	    break;
 	}
 	break;
 	
@@ -1213,13 +1250,13 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    if (verbose)
 		syslog(LOG_DEBUG, 
 		       "service %s pid %d in READY state: serving one more multi-threaded connection",
-		       s->name, c->pid);
+		       SERVICENAME(s->name), c->pid);
 	    break;
 	    
 	case SERVICE_STATE_BUSY:
 	    syslog(LOG_ERR, 
 		   "service %s pid %d in BUSY state: serving one more multi-threaded connection, forced to READY state",
-		   s->name, c->pid);
+		   SERVICENAME(s->name), c->pid);
 	    /* be resilient on face of a bogon source, so lets err to the side
 	     * of non-denial-of-service */
 	    c->service_state = SERVICE_STATE_READY;
@@ -1232,24 +1269,20 @@ void process_msg(struct service *s, struct notify_message *msg)
 	    c->service_state = SERVICE_STATE_READY;
 	    syslog(LOG_ERR,
 		   "service %s pid %d in UNKNOWN state: serving one more multi-threaded connection, forced to READY state",
-		   s->name, c->pid);
-	    break;
-
-	default:
-	    /* Prevent Warning */
+		   SERVICENAME(s->name), c->pid);
 	    break;
 	}
 	break;
 	
     default:
 	syslog(LOG_CRIT, "service %s pid %d: Software bug: unrecognized message 0x%x", 
-	       s->name, c->pid, msg->message);
+	       SERVICENAME(s->name), c->pid, msg->message);
 	break;
     }
 
     if (verbose)
 	syslog(LOG_DEBUG, "service %s now has %d ready workers\n", 
-	       s->name, s->ready_workers);
+	       SERVICENAME(s->name), s->ready_workers);
 }
 
 static char **tokenize(char *p)
@@ -1281,11 +1314,11 @@ static char **tokenize(char *p)
 void add_start(const char *name, struct entry *e,
 	       void *rock __attribute__((unused)))
 {
-    char *cmd = xstrdup(masterconf_getstring(e, "cmd", NULL));
+    char *cmd = xstrdup(masterconf_getstring(e, "cmd", ""));
     char buf[256];
     char **tok;
 
-    if (!cmd) {
+    if (!strcmp(cmd,"")) {
 	snprintf(buf, sizeof(buf), "unable to find command for %s", name);
 	fatal(buf, EX_CONFIG);
     }
@@ -1300,20 +1333,22 @@ void add_start(const char *name, struct entry *e,
 void add_service(const char *name, struct entry *e, void *rock)
 {
     int ignore_err = (int) rock;
-    char *cmd = xstrdup(masterconf_getstring(e, "cmd", NULL));
+    char *cmd = xstrdup(masterconf_getstring(e, "cmd", ""));
     int prefork = masterconf_getint(e, "prefork", 0);
     int babysit = masterconf_getswitch(e, "babysit", 0);
     int maxforkrate = masterconf_getint(e, "maxforkrate", 0);
-    char *listen = xstrdup(masterconf_getstring(e, "listen", NULL));
+    char *listen = xstrdup(masterconf_getstring(e, "listen", ""));
     char *proto = xstrdup(masterconf_getstring(e, "proto", "tcp"));
     char *max = xstrdup(masterconf_getstring(e, "maxchild", "-1"));
     rlim_t maxfds = (rlim_t) masterconf_getint(e, "maxfds", 256);
+    int reconfig = 0;
     int i, j;
+    int provide_uuid = have_uuid && masterconf_getswitch(e, "provide_uuid", 0);
 
     if(babysit && prefork == 0) prefork = 1;
     if(babysit && maxforkrate == 0) maxforkrate = 10; /* reasonable safety */
 
-    if (!cmd || !listen) {
+    if (!strcmp(cmd,"") || !strcmp(listen,"")) {
 	char buf[256];
 	snprintf(buf, sizeof(buf),
 		 "unable to find command or port for service '%s'", name);
@@ -1326,11 +1361,16 @@ void add_service(const char *name, struct entry *e, void *rock)
 	fatal(buf, EX_CONFIG);
     }
 
-    /* see if we have an existing entry for this service */
+    /* see if we have an existing entry that can be reused */
     for (i = 0; i < nservices; i++) {
+	/* skip non-primary instances */
 	if (Services[i].associate > 0)
 	    continue;
-	if (Services[i].name && !strcmp(Services[i].name, name)) break;
+	/* must have empty/same service name, listen and proto */
+	if ((!Services[i].name || !strcmp(Services[i].name, name)) &&
+	    (!Services[i].listen || !strcmp(Services[i].listen, listen)) &&
+	    (!Services[i].proto || !strcmp(Services[i].proto, proto)))
+	    break;
     }
 
     /* we have duplicate service names in the config file */
@@ -1345,146 +1385,98 @@ void add_service(const char *name, struct entry *e, void *rock)
 
 	fatal(buf, EX_CONFIG);
     }
-
-    if ((i < nservices) &&
-	!strcmp(Services[i].listen, listen) &&
-	!strcmp(Services[i].proto, proto)) {
-
-	/* we found an existing entry and the port paramters are the same */
-	Services[i].exec = tokenize(cmd);
-	if (!Services[i].exec) fatal("out of memory", EX_UNAVAILABLE);
-
-	/* is this service actually there? */
-	if (!verify_service_file(Services[i].exec)) {
-	    char buf[1024];
-	    snprintf(buf, sizeof(buf),
-		     "cannot find executable for service '%s'", name);
-
-	    /* if it is not, we're misconfigured, die. */
-	    fatal(buf, EX_CONFIG);
-	}
-
-	Services[i].maxforkrate = maxforkrate;
-
- 	Services[nservices].maxfds = maxfds;
-
-	if (!strcmp(Services[i].proto, "tcp") ||
-	    !strcmp(Services[i].proto, "tcp4") ||
-	    !strcmp(Services[i].proto, "tcp6")) {
-	    Services[i].desired_workers = prefork;
-	    Services[i].babysit = babysit;
-	    Services[i].max_workers = atoi(max);
-	    if (Services[i].max_workers == -1) {
-		Services[i].max_workers = INT_MAX;
-	    }
-	} else {
-	    /* udp */
-	    if (prefork > 1) prefork = 1;
-	    Services[i].desired_workers = prefork;
-	    Services[i].max_workers = 1;
-	}
  
+    if (i == nservices) {
+	/* either we don't have an existing entry or we are changing
+	 * the port parameters, so create a new service
+	 */
+	if (nservices == allocservices) {
+	    if (allocservices > SERVICE_MAX - 5)
+		fatal("out of service structures, please restart", EX_UNAVAILABLE);
+	    Services = xrealloc(Services, 
+			       (allocservices+=5) * sizeof(struct service));
+	}
+	memset(&Services[nservices++], 0, sizeof(struct service));
+
+	Services[i].last_interval_start = time(NULL);
+    }
+    else if (Services[i].listen) reconfig = 1;
+
+    if (!Services[i].name) Services[i].name = xstrdup(name);
+    if (Services[i].listen) free(Services[i].listen);
+    Services[i].listen = listen;
+    if (Services[i].proto) free(Services[i].proto);
+    Services[i].proto = proto;
+
+    Services[i].exec = tokenize(cmd);
+    if (!Services[i].exec) fatal("out of memory", EX_UNAVAILABLE);
+
+    /* is this service actually there? */
+    if (!verify_service_file(Services[i].exec)) {
+	char buf[1024];
+	snprintf(buf, sizeof(buf),
+		 "cannot find executable for service '%s'", name);
+	
+	/* if it is not, we're misconfigured, die. */
+	fatal(buf, EX_CONFIG);
+    }
+
+    Services[i].maxforkrate = maxforkrate;
+    Services[i].maxfds = maxfds;
+    Services[i].provide_uuid = provide_uuid;
+
+    if (!strcmp(Services[i].proto, "tcp") ||
+	!strcmp(Services[i].proto, "tcp4") ||
+	!strcmp(Services[i].proto, "tcp6")) {
+	Services[i].desired_workers = prefork;
+	Services[i].babysit = babysit;
+	Services[i].max_workers = atoi(max);
+	if (Services[i].max_workers == -1) {
+	    Services[i].max_workers = INT_MAX;
+	}
+    } else {
+	/* udp */
+	if (prefork > 1) prefork = 1;
+	Services[i].desired_workers = prefork;
+	Services[i].max_workers = 1;
+    }
+    free(max);
+ 
+    if (reconfig) {
+	/* reconfiguring an existing service, update any other instances */
 	for (j = 0; j < nservices; j++) {
-	    if (Services[j].associate > 0 &&
+	    if (Services[j].associate > 0 && Services[j].listen &&
 		Services[j].name && !strcmp(Services[j].name, name)) {
 		Services[j].maxforkrate = Services[i].maxforkrate;
 		Services[j].exec = Services[i].exec;
 		Services[j].desired_workers = Services[i].desired_workers;
 		Services[j].babysit = Services[i].babysit;
 		Services[j].max_workers = Services[i].max_workers;
+		Services[j].provide_uuid = Services[i].provide_uuid;
 	    }
 	}
-
-	if (verbose > 2)
-	    syslog(LOG_DEBUG, "reconfig: service '%s' (%s, %s:%s, %d, %d)",
-		   Services[i].name, cmd,
-		   Services[i].proto, Services[i].listen,
-		   Services[i].desired_workers,
-		   Services[i].max_workers);
-    }
-    else {
-	/* either we don't have an existing entry or we are changing
-	 * the port parameters, so create a new service
-	 */
-	if (nservices == allocservices) {
-	    Services = xrealloc(Services, 
-			       (allocservices+=5) * sizeof(struct service));
-	}
-
-	Services[nservices].name = xstrdup(name);
-	Services[nservices].listen = listen;
-	Services[nservices].proto = proto;
-	Services[nservices].exec = tokenize(cmd);
-	if (!Services[nservices].exec) fatal("out of memory", EX_UNAVAILABLE);
-
-	/* is this service actually there? */
-	if (!verify_service_file(Services[i].exec)) {
-	    char buf[1024];
-	    snprintf(buf, sizeof(buf),
-		     "cannot find executable for service '%s'", name);
-
-	    /* if it is not, we're misconfigured, die. */
-	    fatal(buf, EX_CONFIG);
-	}
-
-	Services[nservices].socket = 0;
-	Services[nservices].saddr = NULL;
-
-	Services[nservices].ready_workers = 0;
-
- 	Services[nservices].maxfds = maxfds;
-	Services[nservices].maxforkrate = maxforkrate;
-
-	if(!strcmp(Services[nservices].proto, "tcp") ||
-	   !strcmp(Services[nservices].proto, "tcp4") ||
-	   !strcmp(Services[nservices].proto, "tcp6")) {
-	    Services[nservices].desired_workers = prefork;
-	    Services[nservices].babysit = babysit;
-	    Services[nservices].max_workers = atoi(max);
-	    if (Services[nservices].max_workers == -1) {
-		Services[nservices].max_workers = INT_MAX;
-	    }
-	} else {
-	    if (prefork > 1) prefork = 1;
-	    Services[nservices].desired_workers = prefork;
-	    Services[nservices].max_workers = 1;
-	}
-	
-	memset(Services[nservices].stat, 0, sizeof(Services[nservices].stat));
-
-	Services[nservices].last_interval_start = time(NULL);
-	Services[nservices].interval_forks = 0;
-	Services[nservices].forkrate = 0;
-	
-	Services[nservices].nforks = 0;
-	Services[nservices].nactive = 0;
-	Services[nservices].nconnections = 0;
-	Services[nservices].associate = 0;
-	
-	if (verbose > 2)
-	    syslog(LOG_DEBUG, "add: service '%s' (%s, %s:%s, %d, %d, %d)",
-		   Services[nservices].name, cmd,
-		   Services[nservices].proto, Services[nservices].listen,
-		   Services[nservices].desired_workers,
-		   Services[nservices].max_workers,
-		   (int) Services[nservices].maxfds);
-
-	nservices++;
     }
 
-    free(max);
+    if (verbose > 2)
+	syslog(LOG_DEBUG, "%s: service '%s' (%s, %s:%s, %d, %d, %d)",
+	       reconfig ? "reconfig" : "add",
+	       Services[i].name, cmd,
+	       Services[i].proto, Services[i].listen,
+	       Services[i].desired_workers,
+	       Services[i].max_workers,
+	       (int) Services[i].maxfds);
 }
 
 void add_event(const char *name, struct entry *e, void *rock)
 {
     int ignore_err = (int) rock;
-    char *cmd = xstrdup(masterconf_getstring(e, "cmd", NULL));
+    char *cmd = xstrdup(masterconf_getstring(e, "cmd", ""));
     int period = 60 * masterconf_getint(e, "period", 0);
     int at = masterconf_getint(e, "at", -1), hour, min;
     time_t now = time(NULL);
     struct event *evt;
 
-    if (!cmd) {
+    if (!strcmp(cmd,"")) {
 	char buf[256];
 	snprintf(buf, sizeof(buf),
 		 "unable to find command or port for event '%s'", name);
@@ -1573,8 +1565,9 @@ void limit_fds(rlim_t x)
 
 void reread_conf(void)
 {
-    int i;
+    int i,j;
     struct event *ptr;
+    struct centry *c;
 
     /* disable all services -
        they will be re-enabled if they appear in config file */
@@ -1592,17 +1585,26 @@ void reread_conf(void)
 		       Services[i].name, Services[i].socket,
 		       Services[i].stat[0], Services[i].stat[1]);
 
-	    /* Only free the service info once */
+	    /* Only free the service info on the primary */
 	    if(Services[i].associate == 0) {
-		free(Services[i].name);
 		free(Services[i].listen);
 		free(Services[i].proto);
 	    }
-	    Services[i].name = NULL;
+	    Services[i].listen = NULL;
+	    Services[i].proto = NULL;
 	    Services[i].desired_workers = 0;
-	    Services[i].nforks = 0;
-	    Services[i].nactive = 0;
-	    Services[i].nconnections = 0;
+
+	    /* send SIGHUP to all children */
+	    for (j = 0 ; j < child_table_size ; j++ ) {
+		c = ctable[j];
+		while (c != NULL) {
+		    if ((c->si == i) &&
+			(c->service_state != SERVICE_STATE_DEAD)) {
+			kill(c->pid, SIGHUP);
+		    }
+		    c = c->next;
+		}
+	    }
 
 	    /* close all listeners */
 	    if (Services[i].socket > 0) {
@@ -1610,11 +1612,6 @@ void reread_conf(void)
 		close(Services[i].socket);
 	    }
 	    Services[i].socket = 0;
-	    Services[i].saddr = NULL;
-
-	    if (Services[i].stat[0] > 0) close(Services[i].stat[0]);
-	    if (Services[i].stat[1] > 0) close(Services[i].stat[1]);
-	    memset(Services[i].stat, 0, sizeof(Services[i].stat));
 	}
 	else if (Services[i].exec && !Services[i].socket) {
 	    /* initialize new services */
@@ -1627,7 +1624,6 @@ void reread_conf(void)
 	}
     }
 
-
     /* remove existing events */
     while (schedule) {
 	ptr = schedule;
@@ -1641,6 +1637,11 @@ void reread_conf(void)
 
     /* reinit child janitor */
     init_janitor();
+
+    /* send some feedback to admin */
+    syslog(LOG_NOTICE,
+	    "Services reconfigured. %d out of %d (max %d) services structures are now in use",
+	    nservices, allocservices, SERVICE_MAX);
 }
 
 int main(int argc, char **argv)
@@ -1948,6 +1949,12 @@ int main(int argc, char **argv)
 	    exit(1);
 	}
     }
+
+    have_uuid = (config_getint(IMAPOPT_SYNC_MACHINEID) >= 0);
+    if (have_uuid && !message_uuid_master_init()) {
+        syslog(LOG_ERR, "Couldn't initialise UUID subsystem");
+        exit(EX_OSERR);
+    }
     
     /* init ctable janitor */
     init_janitor();
@@ -1979,7 +1986,7 @@ int main(int argc, char **argv)
 	    if (Services[i].exec /* enabled */ &&
 		(Services[i].nactive < Services[i].max_workers) &&
 		(Services[i].ready_workers < Services[i].desired_workers)) {
-		spawn_service(&Services[i]);
+		spawn_service(i);
 	    } else if (Services[i].exec
 		       && Services[i].babysit
 		       && Services[i].nactive == 0) {
@@ -1987,7 +1994,29 @@ int main(int argc, char **argv)
 		       "lost all children for service: %s.  " \
 		       "Applying babysitter.",
 		       Services[i].name);
-		spawn_service(&Services[i]);
+		spawn_service(i);
+	    } else if (!Services[i].exec /* disabled */ &&
+		       Services[i].name /* not yet removed */ &&
+		       Services[i].nactive == 0) {
+		if (verbose > 2)
+		    syslog(LOG_DEBUG, "remove: service %s pipe %d %d",
+			   Services[i].name,
+			   Services[i].stat[0], Services[i].stat[1]);
+
+		/* Only free the service info on the primary */
+		if (Services[i].associate == 0) {
+		    free(Services[i].name);
+		}
+		Services[i].name = NULL;
+		Services[i].nforks = 0;
+		Services[i].nactive = 0;
+		Services[i].nconnections = 0;
+		Services[i].associate = 0;
+                Services[i].provide_uuid = 0;
+
+		if (Services[i].stat[0] > 0) close(Services[i].stat[0]);
+		if (Services[i].stat[1] > 0) close(Services[i].stat[1]);
+		memset(Services[i].stat, 0, sizeof(Services[i].stat));
 	    }
 	}
 
@@ -2071,7 +2100,7 @@ int main(int argc, char **argv)
 		    continue;
 		}
 		
-		process_msg(&Services[i], &msg);
+		process_msg(i, &msg);
 	    }
 
 	    if (Services[i].exec &&
@@ -2081,13 +2110,13 @@ int main(int argc, char **argv)
 		     j < Services[i].desired_workers; 
 		     j++)
 		{
-		    spawn_service(&Services[i]);
+		    spawn_service(i);
 		}
 
 		if (Services[i].ready_workers == 0 && 
 		    FD_ISSET(y, &rfds)) {
 		    /* huh, someone wants to talk to us */
-		    spawn_service(&Services[i]);
+		    spawn_service(i);
 		}
 	    }
 	}
