@@ -1,6 +1,6 @@
 /* message.h
  * Larry Greenfield
- * $Id: message.h,v 1.13 2002/02/19 18:09:46 ken3 Exp $
+ * $Id: message.h,v 1.16.2.2 2005/10/05 15:56:23 ken3 Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -61,6 +61,7 @@ void free_action_list(action_list_t *actions);
    the do_action() functions should copy param */
 struct Action {
     action_t a;
+    int cancel_keep;
     union {
 	sieve_reject_context_t rej;
 	sieve_fileinto_context_t fil;
@@ -72,7 +73,7 @@ struct Action {
 	    sieve_autorespond_context_t autoresp;
 	} vac;
 	struct {
-	    char *flag;
+	    const char *flag;
 	} fla;
     } u;
     char *param;		/* freed! */
@@ -84,13 +85,12 @@ struct Action {
 
 typedef struct notify_list_s {
     int isactive;
-    char *id;
-    char *method;
-    stringlist_t **options;
+    const char *id;
+    const char *method;
+    const char **options;
     const char *priority;
-    char *message;
+    const char *message;
     struct notify_list_s *next;
-
 } notify_list_t;
 
 /* header parsing */
@@ -112,24 +112,24 @@ void free_notify_list(notify_list_t *n);
 /* actions; return negative on failure.
  * these don't actually perform the actions, they just add it to the
  * action list */
-int do_reject(action_list_t *m, char *msg);
-int do_fileinto(action_list_t *m, char *mbox, sieve_imapflags_t *imapflags);
-int do_redirect(action_list_t *m, char *addr);
+int do_reject(action_list_t *m, const char *msg);
+int do_fileinto(action_list_t *m, const char *mbox, int cancel_keep,
+		sieve_imapflags_t *imapflags);
+int do_redirect(action_list_t *m, const char *addr, int cancel_keep);
 int do_keep(action_list_t *m, sieve_imapflags_t *imapflags);
 int do_discard(action_list_t *m);
 int do_vacation(action_list_t *m, char *addr, char *fromaddr,
-		char *subj, char *msg, 
-		int days, int mime);
-int do_setflag(action_list_t *m, char *flag);
-int do_addflag(action_list_t *m, char *flag);
-int do_removeflag(action_list_t *m, char *flag);
+		char *subj, const char *msg, int days, int mime, char *handle);
+int do_setflag(action_list_t *m, const char *flag);
+int do_addflag(action_list_t *m, const char *flag);
+int do_removeflag(action_list_t *m, const char *flag);
 int do_mark(action_list_t *m);
 int do_unmark(action_list_t *m);
-int do_notify(notify_list_t *n, char *id,
-	      char *method, stringlist_t **options,
-	      const char *priority, char *message);
-int do_denotify(notify_list_t *n, comparator_t *comp, void *pat,
-		const char *priority);
+int do_notify(notify_list_t *n, const char *id,
+	      const char *method, const char **options,
+	      const char *priority, const char *message);
+int do_denotify(notify_list_t *n, comparator_t *comp, const void *pat,
+		void *comprock, const char *priority);
 
 
 #endif
